@@ -68,33 +68,52 @@ class BuildCommand(Command):
     def required_system_dependencies_installed(self, module_state):
         '''Returns true if all required system dependencies are installed for
         modules in module_state.'''
-        for pkg_config,(module, req_version, installed_version, new_enough, required_sysdep) in module_state.iteritems():
-            if required_sysdep:
-                if installed_version is None or not new_enough:
+        for module, (req_version, installed_version, new_enough, systemmodule) in module_state.iteritems():
+            if systemmodule:
+                if not new_enough:
                     return False
         return True
 
     def print_system_dependencies(self, module_state):
+
+        def fmt_details(pkg_config, req_version, installed_version):
+            fmt_list = []
+            if pkg_config:
+                fmt_list.append(pkg_config)
+            if req_version:
+                fmt_list.append(_('required=%s') % req_version)
+            if installed_version and installed_version != 'unknown':
+                fmt_list.append(_('installed=%s') % installed_version)
+            # Translators: This is used to separate items of package metadata
+            fmt_str = _(', ').join(fmt_list)
+            if fmt_str:
+                return _('(%s)') % fmt_str
+            else:
+                return ''
+
         print _('Required packages:')
         print _('  System installed packages which are too old:')
         have_too_old = False
-        for pkg_config,(module, req_version, installed_version, new_enough, required_sysdep) in module_state.iteritems():
-            if (installed_version is not None) and (not new_enough) and required_sysdep:
+        for module, (req_version, installed_version, new_enough, systemmodule) in module_state.iteritems():
+            if (installed_version is not None) and (not new_enough) and systemmodule:
                 have_too_old = True
-                print (_("    %(pkg)s (required=%(req)s, installed=%(installed)s)" % {'pkg': pkg_config,
-                                                                                      'req': req_version,
-                                                                                      'installed': installed_version}))
+                print ('    %s %s' % (module.name,
+                                      fmt_details(module.pkg_config,
+                                                  req_version,
+                                                  installed_version)))
         if not have_too_old:
             print _('    (none)')
 
         print _('  No matching system package installed:')
-        uninstalled = []
-        for pkg_config,(module, req_version, installed_version, new_enough, required_sysdep) in module_state.iteritems():
-            if installed_version is None and required_sysdep:
-                print (_("    %(pkg)s (required=%(req)s)") % {'pkg': pkg_config,
-                                                              'req': req_version})
-                uninstalled.append(pkg_config)
-        if len(uninstalled) == 0:
+        have_missing = False
+        for module, (req_version, installed_version, new_enough, systemmodule) in module_state.iteritems():
+            if installed_version is None and (not new_enough) and systemmodule:
+                have_missing = True
+                print ('    %s %s' % (module.name,
+                                      fmt_details(module.pkg_config,
+                                                  req_version,
+                                                  installed_version)))
+        if not have_missing:
             print _('    (none)')
 
 
@@ -127,7 +146,7 @@ def register_command(command_class):
 
 # special help command, never run
 class cmd_help(Command):
-    doc = N_('Information about available jhbuild commands')
+    doc = N_('Information about available JHBuild commands')
 
     name = 'help'
     usage_args = ''
