@@ -47,10 +47,10 @@ class cmd_update(Command):
                         help=_('treat the given modules as up to date')),
             make_option('-t', '--start-at', metavar='MODULE',
                         action='store', dest='startat', default=None,
-                        help=_('start building at the given module')),
+                        help=_('start updating at the given module')),
             make_option('--tags',
                         action='append', dest='tags', default=[],
-                        help=_('build only modules with the given tags')),
+                        help=_('update only modules with the given tags')),
             make_option('-D', metavar='DATE-SPEC',
                         action='store', dest='sticky_date', default=None,
                         help=_('set a sticky date when checking out modules')),
@@ -168,8 +168,8 @@ class cmd_build(BuildCommand):
     def __init__(self):
         Command.__init__(self, [
             make_option('-a', '--autogen',
-                        action='store_true', dest='_unused', default=False,
-                        help=optparse.SUPPRESS_HELP), # no longer used
+                        action='store_true', dest='autogen', default=False,
+                        help=_('always run autogen.sh')),
             make_option('', '--distclean',
                         action='store_true', dest='distclean', default=False,
                         help=_('completely clean source tree')),
@@ -254,7 +254,7 @@ class cmd_build(BuildCommand):
             return 0
 
         if config.check_sysdeps:
-            module_state = module_set.get_system_modules(full_module_list)
+            module_state = module_set.get_module_state(full_module_list)
             if not self.required_system_dependencies_installed(module_state):
                 self.print_system_dependencies(module_state)
                 raise FatalError(_('Required system dependencies not installed.'
@@ -279,8 +279,8 @@ class cmd_buildone(BuildCommand):
     def __init__(self):
         Command.__init__(self, [
             make_option('-a', '--autogen',
-                        action='store_true', dest='_unused', default=False,
-                        help=optparse.SUPPRESS_HELP), # no longer used
+                        action='store_true', dest='autogen', default=False,
+                        help=_('always run autogen.sh')),
             make_option('-c', '--clean',
                         action='store_true', dest='clean', default=False,
                         help=_('run make clean before make')),
@@ -322,6 +322,7 @@ class cmd_buildone(BuildCommand):
         module_set = jhbuild.moduleset.load(config)
         module_list = []
         for modname in args:
+            modname = modname.rstrip(os.sep)
             try:
                 module = module_set.get_module(modname, ignore_case=True)
             except KeyError, e:
@@ -373,35 +374,21 @@ class cmd_run(Command):
                     'command':args[0], 'err':str(exc)})
 
     def run(self, config, options, args, help=None):
-        if options.in_builddir:
+        module_name = options.in_builddir or options.in_checkoutdir
+        if module_name:
             module_set = jhbuild.moduleset.load(config)
             try:
-                module_list = [module_set.get_module(options.in_builddir, ignore_case = True)
-                               for modname in args]
+                module = module_set.get_module(module_name, ignore_case = True)
             except KeyError, e:
                 raise FatalError(_("A module called '%s' could not be found.") % e)
 
-            build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
-            builddir = module_list[0].get_builddir(build)
+            build = jhbuild.frontends.get_buildscript(config, [module], module_set=module_set)
+            if options.in_builddir:
+                workingdir = module.get_builddir(build)
+            else:
+                workingdir = module.get_srcdir(build)
             try:
-                build.execute(args, cwd=builddir)
-            except CommandError, exc:
-                if args:
-                    raise FatalError(_("Unable to execute the command '%s'") % args[0])
-                else:
-                    raise FatalError(str(exc))
-        elif options.in_checkoutdir:
-            module_set = jhbuild.moduleset.load(config)
-            try:
-                module_list = [module_set.get_module(options.in_checkoutdir, ignore_case = True)
-                               for modname in args]
-            except KeyError, e:
-                raise FatalError(_("A module called '%s' could not be found.") % e)
-
-            build = jhbuild.frontends.get_buildscript(config, module_list, module_set=module_set)
-            checkoutdir = module_list[0].get_srcdir(build)
-            try:
-                build.execute(args, cwd=checkoutdir)
+                build.execute(args, cwd=workingdir)
             except CommandError, exc:
                 if args:
                     raise FatalError(_("Unable to execute the command '%s'") % args[0])
