@@ -81,7 +81,7 @@ class cmd_bot(Command):
                         help=_('start as daemon')),
             make_option('--pidfile', metavar='PIDFILE',
                         action='store', dest='pidfile', default=None,
-                        help=_('pid file location')),
+                        help=_('PID file location')),
             make_option('--logfile', metavar='LOGFILE',
                         action='store', dest='logfile', default=None,
                         help=_('log file location')),
@@ -163,6 +163,7 @@ class cmd_bot(Command):
                     config.nonetwork = False
                     phases = ['checkout']
                 elif args[0] == 'build':
+                    config.alwaysautogen = True
                     # make check will be run in another step
                     config.makecheck = False
                     config.build_targets = ['install']
@@ -399,7 +400,7 @@ class cmd_bot(Command):
                 module_list = module_set.get_module_list(
                         self.jhbuild_config.modules,
                         self.jhbuild_config.skip,
-                        include_optional_modules=True)
+                        include_afters=True)
                 config['projects'] = [x.name for x in module_list \
                                       if not x.name.startswith('meta-')]
 
