@@ -45,26 +45,26 @@ _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'installprog', 'repos', 'branches', 'noxvfb', 'xvfbargs',
                 'builddir_pattern', 'module_autogenargs', 'module_makeargs',
                 'interact', 'buildscript', 'nonetwork', 'nobuild',
-                'noinstall', 'makeclean', 'makedistclean', 'makecheck', 'module_makecheck',
-                'use_lib64', 'tinderbox_outputdir', 'sticky_date',
-                'tarballdir', 'pretty_print', 'svn_program', 'makedist',
-                'makedistcheck', 'nonotify', 'notrayicon', 'cvs_program',
-                'checkout_mode', 'copy_dir', 'module_checkout_mode',
-                'build_policy', 'trycheckout', 'min_age',
-                'nopoison', 'module_nopoison', 'forcecheck',
-                'makecheck_advisory', 'quiet_mode', 'progress_bar',
-                'module_extra_env', 'jhbuildbot_master', 'jhbuildbot_slavename',
-                'jhbuildbot_password', 'jhbuildbot_svn_commits_box',
-                'jhbuildbot_slaves_dir', 'jhbuildbot_dir',
-                'jhbuildbot_mastercfg', 'use_local_modulesets',
-                'ignore_suggests', 'modulesets_dir', 'mirror_policy',
-                'module_mirror_policy', 'dvcs_mirror_dir', 'build_targets',
-                'cmakeargs', 'module_cmakeargs', 'print_command_pattern',
-                'static_analyzer', 'module_static_analyzer', 'static_analyzer_template', 'static_analyzer_outputdir',
-                'check_sysdeps',
-                # Internal only keys (propagated from command line options)
-                '_internal_noautogen',
-                ]
+                'alwaysautogen', 'noinstall', 'makeclean', 'makedistclean',
+                'makecheck', 'module_makecheck', 'use_lib64',
+                'tinderbox_outputdir', 'sticky_date', 'tarballdir',
+                'pretty_print', 'svn_program', 'makedist', 'makedistcheck',
+                'nonotify', 'notrayicon', 'cvs_program', 'checkout_mode',
+                'copy_dir', 'module_checkout_mode', 'build_policy',
+                'trycheckout', 'min_age', 'nopoison', 'module_nopoison',
+                'forcecheck', 'makecheck_advisory', 'quiet_mode',
+                'progress_bar', 'module_extra_env', 'jhbuildbot_master',
+                'jhbuildbot_slavename', 'jhbuildbot_password',
+                'jhbuildbot_svn_commits_box', 'jhbuildbot_slaves_dir',
+                'jhbuildbot_dir', 'jhbuildbot_mastercfg',
+                'use_local_modulesets', 'ignore_suggests', 'modulesets_dir',
+                'mirror_policy', 'module_mirror_policy', 'dvcs_mirror_dir',
+                'shallow_clone', 'build_targets', 'cmakeargs', 'module_cmakeargs',
+                'print_command_pattern', 'static_analyzer',
+                'module_static_analyzer', 'static_analyzer_template',
+                'static_analyzer_outputdir', 'check_sysdeps', 'system_prefix',
+                'help_website',
+              ]
 
 env_prepends = {}
 def prependpath(envvar, path):
@@ -185,9 +185,6 @@ class Config:
                 raise FatalError(
                     _('Obsolete JHBuild start script, make sure it is removed '
                       'then do run \'make install\''))
-            
-        # Set defaults for internal variables
-        self._config['_internal_noautogen'] = False
 
         env_prepends.clear()
         try:
@@ -384,6 +381,7 @@ class Config:
             libdir = os.path.join(self.prefix, 'lib')
         self.libdir = libdir
         addpath('LD_LIBRARY_PATH', libdir)
+        os.environ['JHBUILD_LIBDIR'] = libdir
 
         # LDFLAGS and C_INCLUDE_PATH are required for autoconf configure
         # scripts to find modules that do not use pkg-config (such as guile
@@ -623,6 +621,8 @@ class Config:
             options = self.cmdline_options
         else:
             self.cmdline_options = options
+        if hasattr(options, 'autogen') and options.autogen:
+            self.alwaysautogen = True
         if hasattr(options, 'check') and (
                 options.check and not 'check' in self.build_targets):
             self.build_targets.insert(0, 'check')
