@@ -28,6 +28,7 @@ __all__ = [
     ]
 
 import os
+import re
 import shutil
 import logging
 
@@ -81,14 +82,14 @@ def get_dependencies(node):
         for dep in childnode.childNodes:
             if dep.nodeType == dep.ELEMENT_NODE and dep.nodeName == 'dep':
                 typ = dep.getAttribute('type')
-                if not type:
+                if not typ:
                     raise FatalError(_('%(node)s node for %(module)s module is'
                                        ' missing %(attribute)s attribute') % \
                                      {'node_name'   : 'dep',
                                       'module_name' : node.getAttribute('id'),
                                       'attribute'   : 'type'})
                 name = dep.getAttribute('name')
-                if not type:
+                if not name:
                     raise FatalError(_('%(node)s node for %(module)s module is'
                                        ' missing %(attribute)s attribute') % \
                                      {'node_name'   : 'dep',
@@ -142,16 +143,17 @@ def get_branch(node, repositories, default_repo, config):
         try:
             repo = repositories[childnode.getAttribute('repo')]
         except KeyError:
+            repo_names = ', '.join([r.name for r in repositories.values()])
             raise UndefinedRepositoryError(
                 _('Repository=%(missing)s not found for module id=%(module)s. Possible repositories are %(possible)s'
-                  % {'missing': childnode.getAttribute('repo'), 'module': name, 'possible': repositories}))
+                  % {'missing': childnode.getAttribute('repo'), 'module': name,
+                     'possible': repo_names}))
+    elif default_repo:
+        repo = repositories[default_repo]
     else:
-        try:
-            repo = repositories[default_repo]
-        except KeyError:
-            raise UndefinedRepositoryError(
-                _('Default repository=%(missing)s not found for module id=%(module)s. Possible repositories are %(possible)s'
-                  % {'missing': default_repo, 'module': name, 'possible': repositories}))
+        raise UndefinedRepositoryError(
+                _('No repository for module id=%(module)s. Either set branch/repo or default repository.'
+                  % {'module': name}))
 
     if repo.mirrors:
         mirror_type = config.mirror_policy
@@ -180,6 +182,7 @@ class Package:
         self.moduleset_name = None
         self.supports_install_destdir = False
         self.supports_parallel_build = True
+        self.configure_cmd = None
 
     def __repr__(self):
         return "<%s '%s'>" % (self.__class__.__name__, self.name)
@@ -234,7 +237,7 @@ class Package:
         if os.path.isdir(prefixdir):
             self._clean_la_files_in_dir(self, prefixdir)
 
-    def _process_install_files(self, installroot, curdir, prefix):
+    def _process_install_files(self, installroot, curdir, prefix, errors):
         """Strip the prefix from all files in the install root, and move
 them into the prefix."""
         assert os.path.isdir(installroot) and os.path.isabs(installroot)
@@ -250,29 +253,37 @@ them into the prefix."""
             src_path = os.path.join(curdir, filename)
             assert src_path.startswith(installroot)
             dest_path = src_path[len(installroot):]
-            if os.path.isdir(src_path):
-                if os.path.exists(dest_path):
-                    if not os.path.isdir(dest_path):
-                        os.unlink(dest_path)
-                        os.mkdir(dest_path)
-                else:
-                    os.mkdir(dest_path)
-                num_copied += self._process_install_files(installroot, src_path, prefix)
+            try:
                 if os.path.islink(src_path):
+                    linkto = os.readlink(src_path)
+                    if os.path.islink(dest_path) or os.path.isfile(dest_path):
+                        os.unlink(dest_path)
+                    os.symlink(linkto, dest_path)
                     os.unlink(src_path)
+                    num_copied += 1
+                elif os.path.isdir(src_path):
+                    if os.path.exists(dest_path):
+                        if not os.path.isdir(dest_path):
+                            os.unlink(dest_path)
+                            os.mkdir(dest_path)
+                    else:
+                        os.mkdir(dest_path)
+                    num_copied += self._process_install_files(installroot,
+                                                              src_path, prefix,
+                                                              errors)
+                    try:
+                        os.rmdir(src_path)
+                    except OSError:
+                        # files remaining in buildroot, errors reported below
+                        pass
                 else:
-                    os.rmdir(src_path)
-            else:
-                num_copied += 1
-                try:
-                    fileutils.rename(src_path, dest_path)
-                except OSError, e:
-                    logging.error(_('Failed to rename %(src)r to %(dest)r: %(msg)s') %
-                                  {'src': src_path,
-                                   'dest': dest_path,
-                                   'msg': e.message})
-                    raise
-                    
+                    try:
+                        fileutils.rename(src_path, dest_path)
+                        num_copied += 1
+                    except OSError, e:
+                        errors.append("%s: '%s'" % (str(e), dest_path))
+            except OSError, e:
+                errors.append(str(e))
         return num_copied
 
     def process_install(self, buildscript, revision):
@@ -280,7 +291,8 @@ them into the prefix."""
         destdir = self.get_destdir(buildscript)
         self._clean_la_files(buildscript, destdir)
 
-        stripped_prefix = buildscript.config.prefix[1:]
+        prefix_without_drive = os.path.splitdrive(buildscript.config.prefix)[1]
+        stripped_prefix = prefix_without_drive[1:]
 
         previous_entry = buildscript.moduleset.packagedb.get(self.name)
         if previous_entry:
@@ -294,12 +306,14 @@ them into the prefix."""
         save_broken_tree = False
         broken_name = destdir + '-broken'
         destdir_prefix = os.path.join(destdir, stripped_prefix)
+        errors = []
         if os.path.isdir(destdir_prefix):
             destdir_install = True
             logging.info(_('Moving temporary DESTDIR %r into build prefix') % (destdir, ))
-            num_copied = self._process_install_files(destdir, destdir_prefix, buildscript.config.prefix)
-            logging.info(_('Install complete: %d files copied') % (num_copied, ))
-        
+            num_copied = self._process_install_files(destdir, destdir_prefix,
+                                                     buildscript.config.prefix,
+                                                     errors)
+
             # Now the destdir should have a series of empty directories:
             # $JHBUILD_PREFIX/_jhbuild/root-foo/$JHBUILD_PREFIX
             # Remove them one by one to clean the tree to the state we expect,
@@ -356,7 +370,20 @@ them into the prefix."""
                         logging.warn(_("Failed to delete no longer installed file %(file)r: %(msg)s") % { 'file': path,
                                                                                                           'msg': error_string})
 
-            buildscript.moduleset.packagedb.add(self.name, revision or '', absolute_new_contents)
+            buildscript.moduleset.packagedb.add(self.name, revision or '',
+                                                absolute_new_contents,
+                                                self.configure_cmd)
+
+        if errors:
+            raise CommandError(_('Install encountered errors: %(num)d '
+                                 'errors raised, %(files)d files copied. '
+                                 'The errors are:\n  %(err)s') %
+                               {'num'   : len(errors),
+                                'files' : num_copied,
+                                'err'   : '\n  '.join(errors)})
+        else:
+            logging.info(_('Install complete: %d files copied') %
+                         (num_copied, ))
 
     def get_revision(self):
         return self.branch.tree_id()
@@ -470,6 +497,30 @@ them into the prefix."""
             instance.pkg_config = pkg_config
         return instance
 
+class MakeModule(Package):
+    '''A base class for modules that use the command 'make' within the build
+    process.'''
+    def __init__(self, name, branch=None, makeargs='', makeinstallargs='',
+                  makefile='Makefile'):
+        Package.__init__(self, name, branch=branch)
+        self.makeargs = makeargs
+        self.makeinstallargs = makeinstallargs
+        self.makefile = makefile
+
+    def get_makeargs(self, buildscript, add_parallel=True):
+        makeargs = ' %s %s' % (self.makeargs,
+                              self.config.module_makeargs.get(
+                                  self.name, self.config.makeargs))
+        if self.supports_parallel_build and add_parallel:
+            # Propagate job count into makeargs, unless -j is already set
+            if ' -j' not in makeargs:
+                arg = '-j %s' % (buildscript.config.jobs, )
+                makeargs = makeargs + ' ' + arg
+        elif not self.supports_parallel_build:
+            makeargs = re.sub(r'-j\w*\d+', '', makeargs) + ' -j 1'
+        return self.eval_args(makeargs).strip()
+
+
 class DownloadableModule:
     PHASE_CHECKOUT = 'checkout'
     PHASE_FORCE_CHECKOUT = 'force_checkout'
