@@ -23,14 +23,19 @@ __metaclass__ = type
 import os
 import re
 import stat
+try:
+    import hashlib
+except ImportError:
+    import md5 as hashlib
 
 from jhbuild.errors import FatalError, BuildStateError, CommandError
 from jhbuild.modtypes import \
-     Package, DownloadableModule, register_module_type
+     DownloadableModule, register_module_type, MakeModule
+from jhbuild.versioncontrol.tarball import TarballBranch
 
 __all__ = [ 'AutogenModule' ]
 
-class AutogenModule(Package, DownloadableModule):
+class AutogenModule(MakeModule, DownloadableModule):
     '''Base type for modules that are distributed with a Gnome style
     "autogen.sh" script and the GNU build tools.  Subclasses are
     responsible for downloading/updating the working copy.'''
@@ -57,30 +62,18 @@ class AutogenModule(Package, DownloadableModule):
                  autogen_template=None,
                  check_target=True,
                  supports_static_analyzer=True):
-        Package.__init__(self, name, branch=branch)
+        MakeModule.__init__(self, name, branch=branch, makeargs=makeargs,
+                            makeinstallargs=makeinstallargs, makefile=makefile)
         self.autogenargs = autogenargs
-        self.makeargs    = makeargs
-        self.makeinstallargs = makeinstallargs
         self.supports_non_srcdir_builds = supports_non_srcdir_builds
         self.skip_autogen = skip_autogen
         self.skip_install_phase = skip_install_phase
         self.autogen_sh = autogen_sh
-        self.makefile = makefile
         self.autogen_template = autogen_template
         self.check_target = check_target
         self.supports_install_destdir = True
         self.supports_static_analyzer = supports_static_analyzer
 
-    def _get_makeargs(self, buildscript, add_parallel=True):
-        makeargs = self.makeargs + ' ' + self.config.module_makeargs.get(
-            self.name, self.config.makeargs)
-        if self.supports_parallel_build and add_parallel:
-            # Propagate job count into makeargs, unless -j is already set
-            if ' -j' not in makeargs:
-                arg = '-j %s' % (buildscript.config.jobs, )
-                makeargs = makeargs + ' ' + arg
-        return makeargs.strip()
-
     def get_srcdir(self, buildscript):
         return self.branch.srcdir
 
@@ -92,6 +85,83 @@ class AutogenModule(Package, DownloadableModule):
         else:
             return self.get_srcdir(buildscript)
 
+    def _file_exists_and_is_newer_than(self, potential, other):
+        try:
+            other_stbuf = os.stat(other)
+            potential_stbuf = os.stat(potential)
+        except OSError, e:
+            return False
+        return potential_stbuf.st_mtime > other_stbuf.st_mtime
+
+    def _get_configure_cmd(self, buildscript):
+        '''returns a string of the command-line to configure the module.
+        This method may modify self.autogen_sh, if 'autogen.sh' doesn't exist.
+        FIXME: bad idea to modify self.autogen_sh in a getter.
+        '''
+        srcdir = self.get_srcdir(buildscript)
+        if self.autogen_sh == 'autogen.sh' and \
+        not os.path.exists(os.path.join(srcdir, self.autogen_sh)):
+            # if there is no autogen.sh, automatically fallback to configure
+            if os.path.exists(os.path.join(srcdir, 'configure')):
+                self.autogen_sh = 'configure'
+
+        if self.autogen_template:
+            template = self.autogen_template
+        else:
+            template = ("%(srcdir)s/%(autogen-sh)s --prefix %(prefix)s"
+                        " --libdir %(libdir)s %(autogenargs)s ")
+
+        autogenargs = self.autogenargs + ' ' + self.config.module_autogenargs.get(
+                self.name, self.config.autogenargs)
+
+        vars = {'prefix': os.path.splitdrive(buildscript.config.prefix)[1],
+                'autogen-sh': self.autogen_sh,
+                'autogenargs': autogenargs}
+
+        if buildscript.config.buildroot and self.supports_non_srcdir_builds:
+            vars['srcdir'] = self.get_srcdir(buildscript)
+        else:
+            vars['srcdir'] = '.'
+
+        if buildscript.config.use_lib64:
+            vars['libdir'] = "'${exec_prefix}/lib64'"
+        else:
+            vars['libdir'] = "'${exec_prefix}/lib'"
+
+        cmd = self.static_analyzer_pre_cmd(buildscript) + template % vars
+
+        if self.autogen_sh == 'autoreconf':
+            cmd = cmd.replace('autoreconf', 'configure')
+            cmd = cmd.replace('--enable-maintainer-mode', '')
+
+        # Fix up the arguments for special cases:
+        #   tarballs: remove --enable-maintainer-mode to avoid breaking build
+        #   tarballs: remove '-- ' to avoid breaking build (GStreamer weirdness)
+        #   non-tarballs: place --prefix and --libdir after '-- ', if present
+        if self.autogen_sh == 'configure':
+            cmd = cmd.replace('--enable-maintainer-mode', '')
+
+            # Also, don't pass '--', which gstreamer attempts to do, since
+            # it is royally broken.
+            cmd = cmd.replace('-- ', '')
+        else:
+            # place --prefix and --libdir arguments after '-- '
+            # (GStreamer weirdness)
+            if autogenargs.find('-- ') != -1:
+                p = re.compile('(.*)(--prefix %s )((?:--libdir %s )?)(.*)-- ' %
+                       (vars['prefix'], "'\${exec_prefix}/lib64'"))
+                cmd = p.sub(r'\1\4-- \2\3', cmd)
+
+        # If there is no --exec-prefix in the constructed autogen command, we
+        # can safely assume it will be the same as {prefix} and substitute it
+        # right now, so the printed command can be copy/pasted afterwards.
+        # (GNOME #580272)
+        if not '--exec-prefix' in template:
+            cmd = cmd.replace('${exec_prefix}', vars['prefix'])
+
+        self.configure_cmd = cmd
+        return cmd
+
     def skip_configure(self, buildscript, last_phase):
         # skip if manually instructed to do so
         if self.skip_autogen is True:
@@ -108,8 +178,42 @@ class AutogenModule(Package, DownloadableModule):
         if self.skip_autogen == 'never':
             return False
 
-        if buildscript.config._internal_noautogen:
-            return True
+        if buildscript.config.alwaysautogen:
+            return False
+
+        # if autogen.sh args has changed, re-run configure
+        db_entry = buildscript.moduleset.packagedb.get(self.name)
+        if db_entry:
+            configure_hash = db_entry.metadata.get('configure-hash')
+            if configure_hash:
+                configure_cmd = self._get_configure_cmd(buildscript)
+                if hashlib.md5(configure_cmd).hexdigest() != configure_hash:
+                    return False
+            else:
+                # force one-time reconfigure if no configure-hash
+                return False
+        else:
+            # always configure for the first build
+            return False
+
+        # We can't rely on the autotools maintainer-mode stuff because many
+        # modules' autogen.sh script includes e.g. gtk-doc and/or intltool,
+        # which also need to be rerun.
+        # https://bugzilla.gnome.org/show_bug.cgi?id=660844
+        if not isinstance(self.branch, TarballBranch):
+            configsrc = None
+            srcdir = self.get_srcdir(buildscript)
+            for name in ['configure.ac', 'configure.in']:
+                path = os.path.join(srcdir, name)
+                if os.path.exists(path):
+                    configsrc = path
+                    break
+            if configsrc is not None:
+                config_status = os.path.join(self.get_builddir(buildscript),
+                                             'config.status')
+                if self._file_exists_and_is_newer_than(config_status,
+                                                       configsrc):
+                    return True
 
         return False
 
@@ -119,44 +223,15 @@ class AutogenModule(Package, DownloadableModule):
             os.makedirs(builddir)
         buildscript.set_action(_('Configuring'), self)
 
-        srcdir = self.get_srcdir(buildscript)
-        if self.autogen_sh == 'autogen.sh' and \
-        not os.path.exists(os.path.join(srcdir, self.autogen_sh)):
-            # if there is no autogen.sh, automatically fallback to configure
-            if os.path.exists(os.path.join(srcdir, 'configure')):
-                self.autogen_sh = 'configure'
+        cmd = self._get_configure_cmd(buildscript)
 
+        srcdir = self.get_srcdir(buildscript)
         try:
             if not (os.stat(os.path.join(srcdir, self.autogen_sh))[stat.ST_MODE] & 0111):
                 os.chmod(os.path.join(srcdir, self.autogen_sh), 0755)
         except:
             pass
 
-        if self.autogen_template:
-            template = self.autogen_template
-        else:
-            template = ("%(srcdir)s/%(autogen-sh)s --prefix %(prefix)s"
-                        " --libdir %(libdir)s %(autogenargs)s ")
-
-        autogenargs = self.autogenargs + ' ' + self.config.module_autogenargs.get(
-                self.name, self.config.autogenargs)
-
-        vars = {'prefix': buildscript.config.prefix,
-                'autogen-sh': self.autogen_sh,
-                'autogenargs': autogenargs}
-                
-        if buildscript.config.buildroot and self.supports_non_srcdir_builds:
-            vars['srcdir'] = self.get_srcdir(buildscript)
-        else:
-            vars['srcdir'] = '.'
-
-        if buildscript.config.use_lib64:
-            vars['libdir'] = "'${exec_prefix}/lib64'"
-        else:
-            vars['libdir'] = "'${exec_prefix}/lib'"
-
-        cmd = self.static_analyzer_pre_cmd(buildscript) + template % vars
-
         if self.autogen_sh == 'autoreconf':
             # autoreconf doesn't honour ACLOCAL_FLAGS, therefore we pass
             # a crafted ACLOCAL variable.  (GNOME bug 590064)
@@ -166,36 +241,9 @@ class AutogenModule(Package, DownloadableModule):
             extra_env['ACLOCAL'] = ' '.join((
                 extra_env.get('ACLOCAL', os.environ.get('ACLOCAL', 'aclocal')),
                 extra_env.get('ACLOCAL_FLAGS', os.environ.get('ACLOCAL_FLAGS', ''))))
-            buildscript.execute(['autoreconf', '-i'], cwd=srcdir,
+            buildscript.execute(['autoreconf', '-fi'], cwd=srcdir,
                     extra_env=extra_env)
             os.chmod(os.path.join(srcdir, 'configure'), 0755)
-            cmd = cmd.replace('autoreconf', 'configure')
-            cmd = cmd.replace('--enable-maintainer-mode', '')
-
-        # Fix up the arguments for special cases:
-        #   tarballs: remove --enable-maintainer-mode to avoid breaking build
-        #   tarballs: remove '-- ' to avoid breaking build (GStreamer weirdness)
-        #   non-tarballs: place --prefix and --libdir after '-- ', if present
-        if self.autogen_sh == 'configure':
-            cmd = cmd.replace('--enable-maintainer-mode', '')
-            
-            # Also, don't pass '--', which gstreamer attempts to do, since
-            # it is royally broken.
-            cmd = cmd.replace('-- ', '')
-        else:
-            # place --prefix and --libdir arguments after '-- '
-            # (GStreamer weirdness)
-            if autogenargs.find('-- ') != -1:
-                p = re.compile('(.*)(--prefix %s )((?:--libdir %s )?)(.*)-- ' %
-                       (buildscript.config.prefix, "'\${exec_prefix}/lib64'"))
-                cmd = p.sub(r'\1\4-- \2\3', cmd)
-
-        # If there is no --exec-prefix in the constructed autogen command, we
-        # can safely assume it will be the same as {prefix} and substitute it
-        # right now, so the printed command can be copy/pasted afterwards.
-        # (GNOME #580272)
-        if not '--exec-prefix' in template:
-            cmd = cmd.replace('${exec_prefix}', buildscript.config.prefix)
 
         buildscript.execute(cmd, cwd = builddir, extra_env = self.extra_env)
     do_configure.depends = [PHASE_CHECKOUT]
@@ -203,6 +251,8 @@ class AutogenModule(Package, DownloadableModule):
             PHASE_CLEAN, PHASE_DISTCLEAN]
 
     def skip_clean(self, buildscript, last_phase):
+        if 'distclean' in self.config.build_targets:
+            return True
         builddir = self.get_builddir(buildscript)
         if not os.path.exists(builddir):
             return True
@@ -212,7 +262,7 @@ class AutogenModule(Package, DownloadableModule):
 
     def do_clean(self, buildscript):
         buildscript.set_action(_('Cleaning'), self)
-        makeargs = self._get_makeargs(buildscript)
+        makeargs = self.get_makeargs(buildscript)
         cmd = '%s %s clean' % (os.environ.get('MAKE', 'make'), makeargs)
         buildscript.execute(cmd, cwd = self.get_builddir(buildscript),
                 extra_env = self.extra_env)
@@ -221,7 +271,7 @@ class AutogenModule(Package, DownloadableModule):
 
     def do_build(self, buildscript):
         buildscript.set_action(_('Building'), self)
-        makeargs = self._get_makeargs(buildscript)
+        makeargs = self.get_makeargs(buildscript)
         cmd = '%s%s %s' % (self.static_analyzer_pre_cmd(buildscript), os.environ.get('MAKE', 'make'), makeargs)
         buildscript.execute(cmd, cwd = self.get_builddir(buildscript),
                 extra_env = self.extra_env)
@@ -255,7 +305,7 @@ class AutogenModule(Package, DownloadableModule):
 
     def do_check(self, buildscript):
         buildscript.set_action(_('Checking'), self)
-        makeargs = self._get_makeargs(buildscript, add_parallel=False)
+        makeargs = self.get_makeargs(buildscript, add_parallel=False)
         cmd = '%s%s %s check' % (self.static_analyzer_pre_cmd(buildscript), os.environ.get('MAKE', 'make'), makeargs)
         try:
             buildscript.execute(cmd, cwd = self.get_builddir(buildscript),
@@ -268,7 +318,7 @@ class AutogenModule(Package, DownloadableModule):
 
     def do_dist(self, buildscript):
         buildscript.set_action(_('Creating tarball for'), self)
-        makeargs = self._get_makeargs(buildscript)
+        makeargs = self.get_makeargs(buildscript)
         cmd = '%s %s dist' % (os.environ.get('MAKE', 'make'), makeargs)
         buildscript.execute(cmd, cwd = self.get_builddir(buildscript),
                     extra_env = self.extra_env)
@@ -277,7 +327,7 @@ class AutogenModule(Package, DownloadableModule):
 
     def do_distcheck(self, buildscript):
         buildscript.set_action(_('Dist checking'), self)
-        makeargs = self._get_makeargs(buildscript)
+        makeargs = self.get_makeargs(buildscript)
         cmd = '%s %s distcheck' % (os.environ.get('MAKE', 'make'), makeargs)
         buildscript.execute(cmd, cwd = self.get_builddir(buildscript),
                     extra_env = self.extra_env)
@@ -304,14 +354,21 @@ class AutogenModule(Package, DownloadableModule):
         return self.config.noinstall or self.skip_install_phase
 
     def skip_distclean(self, buildscript, last_phase):
-        return self.skip_clean(buildscript, last_phase)
+        builddir = self.get_builddir(buildscript)
+        if not os.path.exists(builddir):
+            return True
+        if not os.path.exists(os.path.join(builddir, self.makefile)) and \
+           not hasattr(self.branch, 'delete_unknown_files'):
+            return True
+
+        return False
 
     def do_distclean(self, buildscript):
         buildscript.set_action(_('Distcleaning'), self)
         if hasattr(self.branch, 'delete_unknown_files'):
             self.branch.delete_unknown_files(buildscript)
         else:
-            makeargs = self._get_makeargs(buildscript)
+            makeargs = self.get_makeargs(buildscript)
             cmd = '%s %s distclean' % (os.environ.get('MAKE', 'make'), makeargs)
             buildscript.execute(cmd, cwd = self.get_builddir(buildscript),
                                 extra_env = self.extra_env)
