@@ -31,7 +31,7 @@ try:
     import xml.dom.minidom
     import xml.parsers.expat
 except ImportError:
-    raise FatalError(_('Python xml packages are required but could not be found'))
+    raise FatalError(_('Python XML packages are required but could not be found'))
 
 from jhbuild import modtypes
 from jhbuild.versioncontrol import get_repo_type
@@ -70,6 +70,7 @@ class ModuleSet:
         self.modules[module.name] = module
 
     def get_module(self, module_name, ignore_case = False):
+        module_name = module_name.rstrip(os.sep)
         if self.modules.has_key(module_name) or not ignore_case:
             return self.modules[module_name]
         module_name_lower = module_name.lower()
@@ -172,6 +173,11 @@ class ModuleSet:
         else:
             module_list = [module for module, after_module in resolved \
                            if not after_module]
+
+        if '*' in skip:
+            module_list = [module for module in module_list \
+                           if module.name in self.config.modules]
+        
         return module_list
 
     def get_test_module_list (self, seed, skip=[]):
@@ -184,37 +190,37 @@ class ModuleSet:
                     test_modules.append(mod)
         return test_modules
 
-    def get_system_modules(self, modules):
-
+    def get_module_state(self, modules):
         installed_pkgconfig = systeminstall.get_installed_pkgconfigs(self.config)
         
-        # pkgconfig -> (required_version, installed_verison)
         module_state = {}
         for module in modules:
-            if (isinstance(module.branch, TarballBranch)
-                or isinstance(module, SystemModule)):
+            # only consider SystemModules or modules with <pkg-config>
+            if (isinstance(module, SystemModule) or
+                (isinstance(module.branch, TarballBranch) and
+                 module.pkg_config is not None)):
                 required_version = module.branch.version
-                if module.pkg_config is None:
-                    module_pkg = module.name
-                else: # Strip off the .pc
+                installed_version = None
+                new_enough = False
+                systemmodule = isinstance(module, SystemModule)
+                if module.pkg_config is not None:
+                    # strip off the .pc
                     module_pkg = module.pkg_config[:-3]
-
-                if module.systemdependencies:
-                    if not systeminstall.systemdependencies_met \
-                               (module.name, module.systemdependencies,
-                                self.config):
-                        module_state[module_pkg] = (module, required_version,
-                                                    None, False,
-                                                    isinstance(module,
-                                                               SystemModule))
-                if module.pkg_config is None:
-                    continue
-                if not module_pkg in installed_pkgconfig:
-                    module_state[module_pkg] = (module, required_version, None, False, isinstance(module, SystemModule))
-                else:
-                    installed_version = installed_pkgconfig[module_pkg]
-                    new_enough = compare_version(installed_version, required_version)
-                    module_state[module_pkg] = (module, required_version, installed_version, new_enough, isinstance(module, SystemModule))
+                    if module_pkg in installed_pkgconfig:
+                        installed_version = installed_pkgconfig[module_pkg]
+                        if required_version is None:
+                            new_enough = True
+                        else:
+                            new_enough = compare_version(installed_version,
+                                                         required_version)
+                elif systemmodule:
+                    new_enough = systeminstall.systemdependencies_met \
+                                     (module.name, module.systemdependencies,
+                                      self.config)
+                    if new_enough:
+                        installed_version = 'unknown'
+                module_state[module] = (required_version, installed_version,
+                                        new_enough, systemmodule)
         return module_state
 
     def remove_system_modules(self, modules):
@@ -279,6 +285,11 @@ class ModuleSet:
             if isinstance(mod, MetaModule):
                 attrs = '[color="lightcoral",style="filled",' \
                         'label="%s"]' % mod.name
+            elif isinstance(mod, SystemModule):
+                label = mod.name
+                if mod.branch.version:
+                    label += '\\n(%s)' % mod.branch.version
+                attrs = '[color="palegreen",style="filled",label="%s"]' % label
             else:
                 label = mod.name
                 color = 'lightskyblue'
