@@ -29,25 +29,28 @@ import cmds
 
 def get_installed_pkgconfigs(config):
     """Returns a dictionary mapping pkg-config names to their current versions on the system."""
-    proc = subprocess.Popen(['pkg-config', '--list-all'], stdout=subprocess.PIPE, env=config.get_original_environment(), close_fds=True)
-    stdout = proc.communicate()[0]
-    proc.wait()
-    pkgs = []
-    for line in StringIO(stdout):
-        pkg, rest = line.split(None, 1)
-        pkgs.append(pkg)
-    # We have to rather inefficiently repeatedly fork to work around
-    # broken pkg-config installations - if any package has a missing
-    # dependency pkg-config will fail entirely.
     pkgversions = {}
-    for pkg in pkgs:
-        args = ['pkg-config', '--modversion']
-        args.append(pkg)
-        proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                                close_fds=True, env=config.get_original_environment())
+    try:
+        proc = subprocess.Popen(['pkg-config', '--list-all'], stdout=subprocess.PIPE, env=config.get_original_environment(), close_fds=True)
         stdout = proc.communicate()[0]
         proc.wait()
-        pkgversions[pkg] = stdout.strip()
+        pkgs = []
+        for line in StringIO(stdout):
+            pkg, rest = line.split(None, 1)
+            pkgs.append(pkg)
+        # We have to rather inefficiently repeatedly fork to work around
+        # broken pkg-config installations - if any package has a missing
+        # dependency pkg-config will fail entirely.
+        for pkg in pkgs:
+            args = ['pkg-config', '--modversion']
+            args.append(pkg)
+            proc = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                    close_fds=True, env=config.get_original_environment())
+            stdout = proc.communicate()[0]
+            proc.wait()
+            pkgversions[pkg] = stdout.strip()
+    except OSError: # pkg-config not installed
+        pass
     return pkgversions
 
 def systemdependencies_met(module_name, sysdeps, config):
@@ -77,8 +80,14 @@ def systemdependencies_met(module_name, sysdeps, config):
             except StopIteration:
                 pass
             return paths
-        # search /usr/include by default
+        try:
+            multiarch = subprocess.check_output(['gcc', '-print-multiarch']).strip()
+        except:
+            multiarch = None
+        # search /usr/include and its multiarch subdir (if any) by default
         paths = [ os.path.join(os.sep, 'usr', 'include')]
+        if multiarch:
+            paths += [ os.path.join(paths[0], multiarch) ]
         paths += extract_path_from_cflags(os.environ.get('CPPFLAGS', ''))
         # check include paths incorrectly configured in CFLAGS, CXXFLAGS
         paths += extract_path_from_cflags(os.environ.get('CFLAGS', ''))
@@ -141,10 +150,23 @@ class SystemInstall(object):
             if possible_cls.detect():
                 return possible_cls()
 
+# PackageKit dbus interface contains bitfield constants which
+# aren't introspectable
+PK_PROVIDES_ANY = 1
+PK_FILTER_ENUM_NOT_INSTALLED = 1 << 3
+PK_FILTER_ENUM_NEWEST = 1 << 16
+PK_FILTER_ENUM_ARCH = 1 << 18
+
 # NOTE: This class is unfinished
 class PKSystemInstall(SystemInstall):
     def __init__(self):
         SystemInstall.__init__(self)
+        self._loop = None
+        # PackageKit 0.8.1 has API breaks in the D-BUS interface, for now
+        # we try to support both it and older PackageKit
+        self._using_pk_0_8_1 = None
+        self._sysbus = None
+        self._pkdbus = None
 
     def _on_pk_message(self, msgtype, msg):
         logging.info(_('PackageKit: %s' % (msg,)))
@@ -152,46 +174,88 @@ class PKSystemInstall(SystemInstall):
     def _on_pk_error(self, msgtype, msg):
         logging.error(_('PackageKit: %s' % (msg,)))
 
-    def install(self, pkgconfig_ids):
-        import dbus
-        import dbus.glib
-        import glib
-
-        loop = glib.MainLoop()
-        
-        sysbus = dbus.SystemBus()
-        pk = dbus.Interface(sysbus.get_object('org.freedesktop.PackageKit',
+    def _get_new_transaction(self):
+        if self._loop is None:
+            import glib
+            self._loop = glib.MainLoop()
+        if self._sysbus is None:
+            import dbus.glib
+            import dbus
+            self._dbus = dbus
+            self._sysbus = dbus.SystemBus()
+        if self._pkdbus is None:
+            self._pkdbus = dbus.Interface(self._sysbus.get_object('org.freedesktop.PackageKit',
                                               '/org/freedesktop/PackageKit'),
                             'org.freedesktop.PackageKit')
-        tid = pk.GetTid()
-        txn = sysbus.get_object('org.freedesktop.PackageKit', tid)
-        txn_tx = dbus.Interface(txn, 'org.freedesktop.PackageKit.Transaction')
+        if self._using_pk_0_8_1 is None:
+            try:
+                txn_path = self._pkdbus.CreateTransaction()
+                txn = self._sysbus.get_object('org.freedesktop.PackageKit', txn_path)
+                self._using_pk_0_8_1 = True
+            except dbus.exceptions.DBusException:
+                tid = self._pkdbus.GetTid()
+                txn = self._sysbus.get_object('org.freedesktop.PackageKit', tid)
+                self._using_pk_0_8_1 = False
+        elif self._using_pk_0_8_1:
+            txn_path = self._pkdbus.CreateTransaction()
+            txn = self._sysbus.get_object('org.freedesktop.PackageKit', txn_path)
+        else:
+            tid = self._pkdbus.GetTid()
+            txn = self._sysbus.get_object('org.freedesktop.PackageKit', tid)
+        txn_tx = self._dbus.Interface(txn, 'org.freedesktop.PackageKit.Transaction')
         txn.connect_to_signal('Message', self._on_pk_message)
         txn.connect_to_signal('ErrorCode', self._on_pk_error)
-        txn.connect_to_signal('Destroy', lambda *args: loop.quit())
+        txn.connect_to_signal('Destroy', lambda *args: self._loop.quit())
+        return txn_tx, txn
 
+    def install(self, uninstalled_pkgconfigs, uninstalled_filenames):
         pk_package_ids = set()
-        txn.connect_to_signal('Package', lambda info, pkid, summary: pk_package_ids.add(pkid))
-        txn_tx.WhatProvides("arch;newest;~installed", "any", map(lambda x: 'pkgconfig(%s)' % (x, ), pkgconfig_ids))
-        loop.run()
 
-        del txn
+        if uninstalled_pkgconfigs:
+            txn_tx, txn = self._get_new_transaction()
+            txn.connect_to_signal('Package', lambda info, pkid, summary: pk_package_ids.add(pkid))
+            if self._using_pk_0_8_1:
+                txn_tx.WhatProvides(PK_FILTER_ENUM_ARCH | PK_FILTER_ENUM_NEWEST |
+                                    PK_FILTER_ENUM_NOT_INSTALLED,
+                                    PK_PROVIDES_ANY,
+                                    ['pkgconfig(%s)' % pkg for modname, pkg in
+                                     uninstalled_pkgconfigs])
+            else:
+                txn_tx.WhatProvides('arch;newest;~installed', 'any',
+                                    ['pkgconfig(%s)' % pkg for modname, pkg in
+                                     uninstalled_pkgconfigs])
+            self._loop.run()
+            del txn, txn_tx
+
+        if uninstalled_filenames:
+            txn_tx, txn = self._get_new_transaction()
+            txn.connect_to_signal('Package', lambda info, pkid, summary: pk_package_ids.add(pkid))
+            if self._using_pk_0_8_1:
+                txn_tx.SearchFiles(PK_FILTER_ENUM_ARCH | PK_FILTER_ENUM_NEWEST |
+                                   PK_FILTER_ENUM_NOT_INSTALLED,
+                                   [pkg for modname, pkg in
+                                    uninstalled_filenames])
+            else:
+                txn_tx.SearchFiles('arch;newest;~installed',
+                                   [pkg for modname, pkg in
+                                    uninstalled_filenames])
+            self._loop.run()
+            del txn, txn_tx
+
+        # On Fedora 17 a file can be in two packages: the normal package and
+        # an older compat- package. Don't install compat- packages.
+        pk_package_ids = [pkg for pkg in pk_package_ids
+                          if not pkg.startswith('compat-')]
 
         if len(pk_package_ids) == 0:
             logging.info(_('Nothing available to install'))
             return
 
-        logging.info(_('Installing: %s' % (' '.join(pk_package_ids, ))))
-
-        tid = pk.GetTid()
-        txn = sysbus.get_object('org.freedesktop.PackageKit', tid)
-        txn_tx = dbus.Interface(txn, 'org.freedesktop.PackageKit.Transaction')
-        txn.connect_to_signal('Message', self._on_pk_message)
-        txn.connect_to_signal('ErrorCode', self._on_pk_error)
-        txn.connect_to_signal('Destroy', lambda *args: loop.quit())
+        logging.info(_('Installing:\n  %s' % ('\n  '.join(pk_package_ids, ))))
 
+        txn_tx, txn = self._get_new_transaction()
         txn_tx.InstallPackages(True, pk_package_ids)
-        loop.run()
+        self._loop.run()
 
         logging.info(_('Complete!'))
 
@@ -203,17 +267,18 @@ class YumSystemInstall(SystemInstall):
     def __init__(self):
         SystemInstall.__init__(self)
 
-    def install(self, pkgconfig_ids):
+    def install(self, uninstalled_pkgconfigs, uninstalled_filenames):
         logging.info(_('Using yum to install packages.  Please wait.'))
 
-        native_packages = []
-        for pkgconfig in pkgconfig_ids:
-            native_packages.append('pkgconfig(' + pkgconfig + ')')
-
-        if native_packages:
-            logging.info(_('Installing: %(pkgs)s') % {'pkgs': ' '.join(native_packages)})
+        if len(uninstalled_pkgconfigs) + len(uninstalled_filenames) > 0:
+            logging.info(_('Installing:\n  %(pkgs)s') %
+                         {'pkgs': '\n  '.join([modname for modname, pkg in
+                                               uninstalled_pkgconfigs +
+                                               uninstalled_filenames])})
             args = self._root_command_prefix_args + ['yum', '-y', 'install']
-            args.extend(native_packages)
+            args.extend(['pkgconfig(%s)' % pkg for modname, pkg in
+                         uninstalled_pkgconfigs])
+            args.extend([pkg for modname, pkg in uninstalled_filenames])
             subprocess.check_call(args)
         else:
             logging.info(_('Nothing to install'))
@@ -227,14 +292,12 @@ class AptSystemInstall(SystemInstall):
     def __init__(self):
         SystemInstall.__init__(self)
 
-    def _get_package_for(self, pkg_config):
-        pattern = '/%s.pc' % (pkg_config, )
-        proc = subprocess.Popen(['apt-file', 'search', pattern],
+    def _get_package_for(self, filename):
+        proc = subprocess.Popen(['apt-file', 'search', filename],
                                 stdout=subprocess.PIPE, close_fds=True)
         stdout = proc.communicate()[0]
         if proc.returncode != 0:
             return None
-        pkg = None
         for line in StringIO(stdout):
             parts = line.split(':', 1)
             if len(parts) != 2:
@@ -248,16 +311,20 @@ class AptSystemInstall(SystemInstall):
             # otherwise for now, just take the first match
             return name
 
-    def install(self, pkgconfig_ids):
+    def install(self, uninstalled_pkgconfigs, uninstalled_filenames):
         logging.info(_('Using apt-file to search for providers; this may be slow.  Please wait.'))
         native_packages = []
-        for pkgconfig in pkgconfig_ids:
-            native_pkg = self._get_package_for(pkgconfig)
+        pkgconfigs = [(modname, '/%s.pc' % pkg) for modname, pkg in
+                      uninstalled_pkgconfigs]
+        for modname, filename in pkgconfigs + uninstalled_filenames:
+            native_pkg = self._get_package_for(filename)
             if native_pkg:
                 native_packages.append(native_pkg)
             else:
-                logging.info(_('No native package found for %(id)s') % {'id': pkgconfig})
-            
+                logging.info(_('No native package found for %(id)s '
+                               '(%(filename)s)') % {'id'       : modname,
+                                                   'filename' : filename})
+
         if native_packages:
             logging.info(_('Installing: %(pkgs)s') % {'pkgs': ' '.join(native_packages)})
             args = self._root_command_prefix_args + ['apt-get', 'install']
