@@ -143,7 +143,9 @@ class GitBranch(Branch):
         self.unmirrored_module = unmirrored_module
 
     def get_module_basename(self):
-        name = os.path.basename(self.module)
+        # prevent basename() from returning empty strings on trailing '/'
+        name = self.module.rstrip(os.sep)
+        name = os.path.basename(name)
         if name.endswith('.git'):
             name = name[:-4]
         return name
@@ -278,9 +280,10 @@ class GitBranch(Branch):
             buildscript.execute(switch_command, cwd=self.get_checkoutdir(),
                     extra_env=get_git_extra_env())
 
-    def pull_current_branch(self, buildscript):
+    def rebase_current_branch(self, buildscript):
         """Pull the current branch if it is tracking a remote branch."""
-        if not self.is_tracking_a_remote_branch(self.get_current_branch()):
+        branch = self.get_current_branch();
+        if not self.is_tracking_a_remote_branch(branch):
             return
 
         git_extra_args = {'cwd': self.get_checkoutdir(),
@@ -292,7 +295,8 @@ class GitBranch(Branch):
             buildscript.execute(['git', 'stash', 'save', 'jhbuild-stash'],
                     **git_extra_args)
 
-        buildscript.execute(['git', 'pull', '--rebase'], **git_extra_args)
+        buildscript.execute(['git', 'rebase', 'origin/' + branch],
+                            **git_extra_args)
 
         if stashed:
             # git stash pop was introduced in 1.5.5,
@@ -391,17 +395,22 @@ class GitBranch(Branch):
 
     def _checkout(self, buildscript, copydir=None):
 
+        extra_opts = []
         if self.config.quiet_mode:
-            quiet = ['-q']
-        else:
-            quiet = []
+            extra_opts.append('-q')
+
+        if self.config.shallow_clone:
+            extra_opts.append('--depth=1')
 
         self.update_dvcs_mirror(buildscript)
 
-        cmd = ['git', 'clone'] + quiet + [self.module]
+        cmd = ['git', 'clone'] + extra_opts + [self.module]
         if self.checkoutdir:
             cmd.append(self.checkoutdir)
 
+        if self.branch is not None:
+            cmd.extend(['-b', self.branch])
+
         if copydir:
             buildscript.execute(cmd, cwd=copydir, extra_env=get_git_extra_env())
         else:
@@ -434,7 +443,7 @@ class GitBranch(Branch):
 
         self.switch_branch_if_necessary(buildscript)
 
-        self.pull_current_branch(buildscript)
+        self.rebase_current_branch(buildscript)
 
         self._update_submodules(buildscript)
 
@@ -448,6 +457,10 @@ class GitBranch(Branch):
             raise CommandError(_('%s not found') % 'git')
         Branch.checkout(self, buildscript)
 
+    def delete_unknown_files(self, buildscript):
+        git_extra_args = {'cwd': self.get_checkoutdir(), 'extra_env': get_git_extra_env()}
+        buildscript.execute(['git', 'clean', '-d', '-f', '-x'], **git_extra_args)
+
     def tree_id(self):
         if not os.path.exists(self.get_checkoutdir()):
             return None
